// RELATED SOFTWARE: Scilab >= 5.5.0
// PACKAGE  : uman : Unified user MANual for Scilab and external modules
// VERSION  : 1.01 - 2015-03-27
// AUTHOR   : Samuel GOUGEON - private individual
// COPYRIGHT: Samuel GOUGEON - 2015
// LICENSE for uman : CeCILL : http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// DISCLAIMER: This toolbox is provided as is, with NO WARRANTY of any kind.
//   Use it to your own risks & responsability.
//
// TODO :
//  - "r": only the page in the queried language was re-extracted, not in all languages
//  - localization: tbx_generate_pofile("uman", pwd()) does not work (no output)

function uman(varargin)
// 
//uman - User manual in console, online, or browser, with language switch & related bugs
// 
//CALLING SEQUENCES
//-----------------
// uman
// uman pattern
// uman pattern options
// uman -options pattern
// uman -options pattern options
// 
//PARAMETERS
//----------
//  pattern : a single string: function name, symbol ($, [, etc ), expression...
//            about which informations are queried. The query is performed in a 
//            case-insensitive way if the pattern is all in lowercase. It is 
//            case-sensitive otherwise. To include white spaces, coma or 
//            semi-columns, embrasse the pattern between quotes "..".
//  
//  options : Options are described in details within the /Description/ section.
//            Single string specifying one or several options. Each character
//            (order and case unsensitive) is a switch enabling a specific
//            feature. Available options are described here-above.
//            WHERE it must be displayed. By default, the text output is displayed
//            in the console. Other targets are possible:
//             g : Help browser GUI. Display in console is still done.
//             w : Web page: display the related online help or bugs page in your
//                 web browser. Display in console is canceled.
//             
//            HOW the query must be processed:
//             c   : Clear the Console before displaying the page
//             L## : Language of the version to be displayed (if not the session's
//                   one). ## = language code en | fr | de | ja | pt | ru | zh
//                   (default specified as last option = "en").
//             
//             x   : priority to eXternal packages and references.
//             r   : Refresh / Reload the list of all available modules
//                   documentations, and the queried page.
//             
//             j   : Journalize: allows recording the output in currently open
//                   diaries (if any).
//             
//            WHAT must be displayed. By default, the pattern help page is
//            displayed, with the following sections: Related help section /
//            Actual name of the feature / Calling sequences / List of input &
//            output parameters / /See also/ section. Additional sections
//            displayed on query:
//             s : summary: display the parent summary including the pattern,
//                 instead of its page.
//             
//             a : All sections displayed, but bugs URLs. Equivalent to "deh"
//                 options.
//             
//             d : Description: display /Description, Bibliography, References,
//                 Authorship/ sections, as well as any other sections of
//                 unidentified types.
//                 NOTE: When the pattern includes "_properties", the description is
//                 always displayed.
//             
//             e : Examples: display identified Examples section(s)
//             h : History
//             b : Bugs: URL of the web page listing documented bugs related to the
//                 pattern. When "w" is also used, the page is opened in the web browser.
//             
// 
//DESCRIPTION
//-----------
// /You need human help? Help yourself: Just call uman !/
// uman.. is a Unified User MANual. It displays help pages of Scilab, ATOMS
// packages, external packages, or heading comments in macros, in text mode in
// the console, as well as in the help browser or online in your web browser
// (when applicable). A single function for every sources and displayers, with
// advanced features such that language switching, summaries, or display of bugs
// related to the query.
//
//  Main features : 
//         
//     * Help pages from Scilab, ATOMS modules, other external contribs, and
//       heading comments in functions are supported. No need to priorly load
//       modules in session to view their pages in console.
//       
//     * Adaptative depth of infos: focus on main infos: path in help, calling
//       sequences, parameters, see also. Be free to display one or more
//       additional sections on request: description, examples, history...
//       
//     * Language switch without changing the session's language. Just give the
//       language code en | de | fr | ja | pt | ru | zh in option, and you get
//       the related page! Thus, access easily to the reference en_US version of
//       the page.
//       
//     * Smart text layout in console. Nested lists of items, tables, pieces of
//       codes, notes and warnings, text styles.. are supported. Improved code
//       style of calling sequences and examples.
//       
//     * Almost 200 automatic redirections for users coming from other
//       softwares: Search for atan2 leads to.. atan.
//     * Easy switch to online pages, with redirection and language facilities,
//       to see users comments or to straight comment the right page.
//     * Check the forge: easy and ergonomic access to the list of bugs related
//       to the pattern, for Scilab and many ATOMS packages.
//
//  Supported OS : Windows, Linux and Mac OS, with Scilab 5.5.0 or later.
//
//  Covered help pages : 
//     Scilab pages : are firstly and fully covered, in the console as well as online.
//     ATOMS pages : 
//          If some ATOMS modules have been installed for your Scilab, either
//          for /all users/ (by an admin) or only for you (by yourself), all
//          their pages are automatically considered.
//          NOTE: ATOMS modules do not need to be loaded in session, just be
//          installed (in SCI\.atoms and SCI\contrib or SCIHOME\atoms\..). It is
//          therefore possible to have a look at some pages before loading the
//          related module, whether it looks to be the relevant one.
//          
//          NOTE: To target eXternal pages from ATOMS modules or other contrib
//          modules in priority, before Scilab pages, use the "x" option.
//     
//     Other contrib packages : 
//          Every package installed by hand in SCI\contrib or in SCIHOME\contrib
//          and having standard help pages are also covered. As for ATOMS modules,
//          NOTE: To display their documentation in the console, external contrib
//          modules do not need to be loaded in session, just to be installed (in
//          SCI\contrib\.. or SCIHOME\contrib\..).
//          
//          WARNING: Pages from Contributed external packages are scanned only if
//          the pattern has not been found in the main Scilab pages, unless the "x"
//          option is used.
//     
//     Redirections : 
//          If uman does not find any matching item in Scilab and in installed
//          ATOMS and contrib packages, it silently looks whether a redirection
//          is defined for the queried pattern. Redirections are defined for
//          some usual terms existing in other main Scientific languages, that
//          do not exist in Scilab, and that do not mean anything else in Scilab
//          (faux amis). Some helping terms are also processed, such as typos
//          like lenght for length... If a redirection matches the query, uman
//          is rerun with it.
//     
//     Miscellaneous macros : 
//          If definitively uman does not find any matching item in Scilab and
//          installed ATOMS and contrib packages, even within a redirection, it
//          looks whether the pattern is a macro defined in the session. If it
//          is the case, it displays its eventual header comments, the easiest
//          way to document a macro.
//
//  Options : single string specifying one or several options. Each character
//        (order and case unsensitive) is a switch enabling a specific
//        feature. Available options are described here-above.
//        By default, options are specified /after/ the pattern. Otherwise,
//        "-" must prefix the options string.
//        Options specified before AND after the pattern are cumulated.
//        Frequently used options may be used /before/ the pattern, while it
//        will be more handy changing versatile ones in end of command line.
//        HOW the query must be processed:
//         c   : Clear the Console before displaying the page
//         L## : Language: gets the version of the page in Language ##, where
//               ## stands for one of the en | fr | de | ja | pt | ru | zh
//               language codes. Otherwise, the session's language is used. If
//               L is specified as last option without ## code, or if the
//               specified code is out of this list, the reference version "en"
//               is used.
//         
//         x   : eXternal packages and references first. This flag is used in 2
//               different ways:
//                
//                * When the same function name is used in Scilab and in an
//                  ATOMS or a contrib package, by default uman addresses the
//                  Scilab version. To target the eXternal package version
//                  first, use "x". If "x" is used while there is no version of
//                  the pattern in external packages, Scilab is finally targeted.
//                  
//                * When the same pattern exists in Scilab and in an eXternal
//                  scientific language /with 2 different meanings/, by default
//                  the Scilab meaning is considered, and the related Scilab
//                  page is targeted. If the eXternal meaning must be
//                  considered, use "x". Then, uman will redirect the user
//                  toward the /equivalent/ Scilab page.
//                  Faux-amis examples: type, end, load, home, null ...
//                  
//         r   : Refresh / Reload the list of all available modules
//               documentations, and the queried page.
//               By default, the list of documentations is initially set,
//               stored, and automatically refreshed once a week. In
//               addition, each new page extracted from the help system is
//               stored in cache. Any further call to this page uses the stored
//               version. After having installed some new modules, it is
//               necessary to refresh uman to detect their help pages. As well,
//               after updating an help page of a module, it is necessary to
//               delete its stored version and to extract again the page. This
//               is what "r" does.
//         
//         j   : Journalize: records the output in currently opened diaries (if
//               any).
//               By default, opened diaries are paused before and resumed after
//               the "uman" flows, in order to not be fed. If no diary is
//               priorly open, no journalization is done.
//         
//        WHAT must be displayed.
//        By default, the help page matching the query is displayed. However,
//        the s switch allows to display instead the parent Summary to which
//        the pattern belongs, = the list of functions or features of the same
//        type, appearing at the same level.
//        When an help page is displayed, the mandatory sections are the
//        following: Path through the documentation, leading to the feature /
//        Actual name of the feature / Calling sequences / List of input &
//        output parameters / "See also" section. Additionnal sections may be
//        displayed on query:
//         d : Description: display also description sections. This includes
//             /Bibliography, References, Authorship/ sections, whether they
//             exist. Any other sections whose types are unidentified are also displayed.
//             NOTE: When the pattern includes "_properties", the description is
//             always displayed.
//         
//         e : Examples: display also identified Examples section(s)
//             Examples included in other types of sections are displayed
//             within these ones, without requiring the "e" option.
//         
//         h : History: display also the History section, whether it exists
//         b : Bugs: displays links about bugs reporting / reported. When both
//             "b" and "w" are used, the online page reporting bugs about the
//             pattern is opened instead (provided that the user has a
//             persistent internet connection).
//             If the pattern is related to an installed ATOMS module:
//              
//              * If this module has a Scilab's forge at
//                http://forge.scilab.org, the ticket page of its forge is
//                opened in your web browser and filtered with the pattern.
//                
//              * Else, if this module has a declared web site, its home page is
//                opened in your web browser.
//              * Otherwise, the ATOMS web page of the module is opened. Its
//                comments section is targeted. This is the first place to get
//                help about the module or to report its pitfalls.
//                
//         a : All sections displayed, but bugs URLs. Option equivalent to "deh".
//         s : Summary: display the parent summary including the pattern,
//             instead of its page.
//             NOTE: This option works also with the "g" one (simultaneous output
//             in the Help browser GUI).
//             NOTE: This option kills "w" and "b" ones (online and bug addressing
//             cancelled).
//             WARNING: Options adding sections are then ignored: "a", "b", "d",
//             "e", "h",...
//             WARNING: This option is ignored when "w" is used (except for pages
//             of external packages).
//         
//        WHERE it must be displayed.
//        By default, the text output is displayed in the console. Other
//        targets are possible:
//         g : Help browser GUI: simultaneously calls the Scilab help browser
//             for the same pattern. If the browser is not already opened, it
//             is called in the active or forced language (see the "L##" option).
//             WARNING: If the pattern belongs to an external package installed
//             but not loaded, related help pages can't be available in the help
//             browser. This warning is then displayed in the console.
//             
//             WARNING: When "g" is used with "x" (eXternal packages), the page
//             displayed in the browser is the Scilab version (if any) of the
//             pattern, not the external one. There is no way to force the browser
//             considering first or only non-Scilab pages.
//         
//         w : Web page: opens your web browser and displays the help page of
//             the online manual, for the given pattern (internet connection
//             alive required). The language (default, or forced with the "l"
//             switch) is taken into account. Page's display in the console is
//             then cancelled.
//             If the pattern is not found in the help, the search engine of
//             the online help is automatically targeted with it.
//             When "w" is used with "b", the online manual page is replaced
//             with the online bug tracker page related to the pattern -- when
//             it exists.
//             When the pattern is related to an external module (from or out
//             of the ATOMS external modules repository), no online help pages
//             are available (see http://bugzilla.scilab.org/6723). This "w" is
//             then ignored, and the help page is as usual displayed in text
//             mode in the console.
//
//  Shortcuts : 
//            The following patterns (in lower case) do not exist in Scilab but
//            are defined to display related contents or summaries in a
//            comprehensive way: COLORS(colors page), DATATYPE (Data types page),
//            FILES (summary of file management functions), GRAPHICS (summary),
//            HDF5 (summary of functions), LINESPEC (related page), METANET
//            (summary), SHORTCUTS (related page), SPARSES (summary of main
//            functions for sparse matrices), STATISTICS (summary), VARIABLES
//            (summary), WINDOWS (summary), XML (summary).
//            Try: -->umansparses and see.
//  
//  Technical : 
//
//     * uman extracts Scilab pages in the SCIHOME\uman directory, and
//       ATOMS and contrib pages in \SCIHOME\uman\atoms one.
//     * contrib packages:
//        
//        * Each package must have a ~\help\* directory containing
//          scilab_##_##_help.jar standard help zipped files, where ##_##
//          stands for each supported language (one file per language). At
//          least, a reference scilab_en_US_help.jar file must be available.
//          
//        * The package must also have a ~\etc\<package_name>.start
//          directory and file. The file may be empty. Only its name
//          <package_name> is used: it is and it provides the package's name.
//          
//        * Each contrib package may be stored anywhere in the SCI\contrib
//          or SCIHOME\contrib subtrees, not necessarily at their roots.
//
//  Known limitations : 
//             
//     * Each week, the first call to uman lasts a bit before answering.
//       This time is spent to detect and reference all available help pages on
//       your computer.
//     * Switching language in the help browser with the "g" option needs
//       first to close it (by hand ; no way by code)
//     * Calling the help browser with the "g" option does not bring it to
//       the foreground: see http://bugzilla.scilab.org/13756
//     * LaTeXed expressions rendered as images can't be reversed and
//       displayed as clear text.
//     * Long lines in multicolumn table's cells are not wrapped.
//     * In Japanese and Chinese, long lines without space are not wrapped.
//     * Over more than 2000 help pages, a few of them are mis-conditionned
//       and may mislead uman. Reconditionning these pages is softly going
//       on ;)
//       
//     * Presently, uman is not self-extensible to any language. If you
//       wish to add a (left-to-right) language, please get in touch with
//       the author
//       
//     * Particular cases : for uman " = " .., uman "==", uman "(...",
//       uman "//"..., quotes are mandatory.
//     * uman can't be used in /console/ mode for MS Windows releases (UTF8
//       badly rendered: See http://bugzilla.scilab.org/10748).
//
// 
//EXAMPLES
//--------
// uman                // display this man's help (main contents)
// 
// uman .*. cd         // Operators or symbols are accepted. here: shortcut to kron()
// uman  $  cde        // Another one. Add examples.
// 
// uman linspace Lrua  // Russian version of linspace's page
// uman linspace aL    // English reference version
// 
// uman uint16  ce     // Pages presenting several functions are supported (here int8, int16, etc)
// 
// // nested itemized or unordered lists are supported
// uman daskr cd   
// //     Let's decrease the console's width. Then re-run
// uman daskr cd       // Wrapping is adjusted (but neither for the code samples)
// 
// // Tables are supported :
// uman PlotSparse cd   // 2 tables without borders, in the arguments section
// uman overloading cd  // tables with borders.
// 
// // Inner redirections to pages or summaries :
// uman colors      // inner redirection to the color_list page,with its Description
// uman sparses     // Display the summary of functions dealing with sparse matrices. 
//                  // Other shortcuts are defined in the same way. Try with "statistics"
//                  
// // Scilab equivalences of external patterns inexisting in Scilab
// uman polyval c   // Automatic Octave => Scilab redirection performed when possible.
//                  // polyval() does not exist in Scilab but is the Octave
//                  // function for Scilab's horner() => horner's page is displayed
//                   
// // Scilab equivalence of external faux-amis			  
// uman end cde     // targets the Scilab related page = controls (if.. end / for.. end / while..end)
// uman end cdex    // "x" targets in priority an eXternal meaning for "end". In Octave, "end" 
//                  // means "index of last element", as "$" in Scilab => "$" page is displayed
//                   
// // "g" calls also the help browser GUI (may be docked over the editor)
// uman strstr deg
// 
// // "s" display the related Summary instead of the feature's page
// uman strstr des
// uman who sg     // A Summary with subcategories, here in the browser as well
// 
// // With an internet connexion alive:
// uman ode wlpt   // ONLINE page of ode() in Portuguese
// uman ode wb     // ONLINE list of bugs reported around ode(), well sorted (unfixed bugs first)
// uman colors w   // inner redirection to color_list, then ONLINE page (with colors)
// 
// // With an ATOMS package. 
// uman atoms d           // List of ATOMS functions
// yn = atomsIsInstalled("plotlib"); 
// atomsInstall plotlib ; // Let's install Plotlib
// atomsIsLoaded plotlib  // => %F
// uman plot cde       // the Scilab's plot() page is targeted in priority
// uman plot cdex      // "x" for the eXternal plot() page from Plotlib to be targeted
// uman plot cdexg     // the help browser called with "g" does not come, because Plotlib is not loaded
// uman quiver wb      // targets the ONLINE bug page of the plotlib module on the Scilab forge.
// if ~yn, atomsRemove("plotlib"), end    // cleaning after the example
// 
//SEE ALSO
//--------
//  help
//  apropos
//  head_comments
//  tbx_build_help
//  xmltojar
// 
//HISTORY
//-------
//  Version          Description  
//  1.0 / scilab 5.6 First release


// CONSTRAINING EXTERNAL BUGS :
//   html parsing: http://bugzilla.scilab.org/12748 (worked around) 
//   parsing <pre>: http://bugzilla.scilab.org/show_bug.cgi?id=12588 (worked around)
//   strsubst: http://bugzilla.scilab.org/12808  (worked around)
//   diary: http://bugzilla.scilab.org/show_bug.cgi?id=12551 (ignored)

// --------------------------------------------------------------------------

    // FORMATING and partially PARSING OPTIONS
    // ---------------------------------------
    if argn(2)==0 then
        uman("uman","e")
        return
    end
    if argn(2)==3 then
        tmp = convstr(varargin(3))
        if grep(tmp,"l")~=[]
            varargin(1) = varargin(1)+tmp
        else
            varargin(1) = "-"+tmp+part(varargin(1), 2:$)
        end
    end
    if argn(2)==1 then
        if varargin(1)=="-"  then
            fun0 = "-"
            options = ""
        elseif part(varargin(1),1)=="-"
            if grep(varargin(1),"c")~=[] then
                clc
            end
            uman("uman")
            return
        else
            fun0 = varargin(1)
            options = ""
        end
    else
        if varargin(1)=="-" | part(varargin(1),1)~="-" then
            fun0 = varargin(1)
            options = varargin(2)
        else
            fun0 = varargin(2)
            options = varargin(1)
        end
    end
    
    // OPTIONS ANALYSIS
    // ----------------
    options0 = convstr(options)  // useful in case of redirection (recursive => call)
    i = strindex(options0, "l")
    if i~=[] then
        LANG    = part(options0, (i(1)+1):(i(1)+2))
        if LANG==""
            LANG = "en"
        end
        options = uman_strsubst(options, ["/l../i" "/l$/i"], ["" ""], "r")
    else
        LANG = part(getlanguage(), 1:2)
    end
    
    fun = fun0
    if grep(options, "a")~=[] then
        options = strsubst(options,"a","deh")
    end
    if grep(options, "x")~=[] then
        fun  = redirect_from_external(fun, options)
        // Some equivalences have "£" inserted: prefixing chars are options to force
        //  By default, "s" is forced, meaning that the equivalence is designed
        //  to target a parent summary:
        if grep(fun,"£")~=[] then
            s = strtok(fun, "£")
            fun = part(strchr(fun,"£"), 2:$)
            if s==fun
                if grep(options, "s")==[]
                    options = "s" + options
                end
            else
                options = s + options
            end
        end
    end
    fun = uman_strsubst(fun,"/^%/" ,"percent","r") 
    // This substitution will have to be cancelled in "wb" mode before calling
    //  the bug page. Nevertheless, it is required to search the right page
    //  to determine the related module (or scilab) and the related bug page 
    //  to be opened.
    
    // option "s" (summary) kills option "b" (bug)
    if grep(options, "s")~=[] then
        options = strsubst(options, "b", "")
    end
    bugOnWeb = grep(options, "w")~=[] & grep(options,"b")~=[]

    // SETTING THE LANGUAGE
    // --------------------
    supportedLANG = [ "en_US" "fr_FR" "ja_JP" "de_DE" "pt_BR" "ru_RU" "zh_CN"]
    i = grep(supportedLANG, "/^"+LANG+"_/", "r")
    if i==[] then
        LANG = "en_US"
    else
        LANG = supportedLANG(i)
    end
    
    // REDIRECTIONS for Symbols. Translation for "%" in fun names
    // ----------------------------------------------------------
    if ~bugOnWeb then
        fun = uman_strsubst(fun,["/\{/" "/\}/"],["[" "]"],"r") 
        sub = "assignation,great,  > ,  < ,   [    ,   ]    ,  :  ,   //   ,"
        ids = "insertion  ,less ,less,less,brackets,brackets,colon,comments,"
        
        sub = sub+"    <>    ,   ~=     ,   <=     ,   >=     ,"
        ids = ids+"comparison,comparison,comparison,comparison,"

        sub = sub+"  =  ,    ==    ,"
        ids = ids+"equal,comparison,"
        // bug with these ones, due to Scilab parser. 
        // Must be set within quotes: uman "=="
        
        sub = sub+"  &   ,  |  , ~ ,   $  , . , []  ,"
        ids = ids+"and_op,or_op,not,dollar,dot,empty,"
            // % automaticaly replaced with percent 
        
        sub = sub+"  -  ,  + ,  * , .* ,  /  , ./  ,  /. , .^  , **  , ^ , .*.,    \    ,"
        ids = ids+"minus,plus,star,star,slash,slash,slash,power,power,hat,kron,backslash,"
        
        sub = sub+"    \.     ,     *.    ,    ./.    ,    .\.    ,"
        ids = ids+"overloading,overloading,overloading,overloading,"
        
        sub = sub+"  '' ,    ;    "
        ids = ids+"quote,semicolon"
        
        sub = sub+""
        ids = ids+""
        sub = [ stripblanks(strsplit(sub, ",")) ; "," ]
        ids = [ stripblanks(strsplit(ids, ",")) ; "comma" ]
        i = find(fun==sub)
        if i~=[] then
            fun = ids(i(1))
        end
        if grep(fun, " ") then
            fun = strsubst(fun, " ", "_")
        end
    end
    
    // If the query includes "property" or "properties", "d" description flag 
    //  is made mandatory
    if grep(convstr(fun),"propert")~=[] & grep(options, "d")==[] then
        options = "d" + options
    end

    // OTHER INITIALISATIONS
    // ---------------------
    // Case sensitivity
    if convstr(fun)==fun then
        casse = "-C"
    else
        casse = ""
    end
    // unzip application vs OS
    if getos() == "Windows" then
        unzip  = pathconvert(SCI+"\tools\zip\") + "unzip.exe "+casse+" -o " 
        //unzip2 = pathconvert(SCI+"\tools\gzip\") + "gzip.exe "
    else
        unzip = "unzip "+casse+" "
    end

    fs = filesep()
    global uman_alreadyWebed uman_HeadCommented
    uman_alreadyWebed = %f
    uman_HeadCommented = %f    
    htmlFound = %f
    unzippedFile = ""        // = flag for "File Not Found"

    manDir = pathconvert(SCIHOME + "\uman\")   // storage of unzipped HTML files
    if fileinfo(manDir)==[] then  
        mkdir(manDir)
    end
    manAtomsDir = pathconvert(SCIHOME + "\uman\atoms\")  // storage of unzipped HTML files
    if fileinfo(manAtomsDir)==[] then  
        mkdir(manAtomsDir)
    end
    mapDir = manDir + "jhelpMaps" + fs          // storage of jhelpmap.jhm files
    if ~isdir(mapDir) then
        mkdir(mapDir)
    end

    subDirEN   = "scilab_en_US_help"
    subDirLANG = "scilab_" + LANG + "_help"

    atoms = atomsGetInstalled()
    if atoms~=[] then
        atoms(:,4) = getshortpathname(pathconvert(atoms(:,4)))    
    end
    module = ""
    
    // SEARCHING FOR THE RIGHT HELP FILE. MANAGING UNAVAILABLE ONE. ZIP EXTRACTION
    // ===========================================================================
    // We need this part also for bugOnWeb: if the pattern matches an external
    //  function (e.g. from ATOMS), the external module must be identified
    //  in order to target the right online bug report place
    jarListFile = manDir+"jarListe.txt"
    i = fileinfo(jarListFile)
    // If the jar list does not yet exist, or if it is more than 7 days old,
    //  or if the "r" (reload / refresh) option is used, we rebuild and record 
    //  it and re-extract jhelpmap.jhm files :
    if i==[] | (getdate("s")-i(6))>7*24*3600 | grep(options,"r")~=[] then
        jarList = uman_listJar()
        csvWrite(jarList, jarListFile, ascii(9))
        extractJhelpMaps(jarList, atoms, unzip, manDir, mapDir)
    else
        jarList = mgetl(jarListFile)
    end
    
    // Looking for the pattern in jhelpmap files 
    //  => related jar & html filename
    // -----------------------------------------
    // Sorting jhm files to be scanned
    jhmTab = listfiles(mapDir)     // jhmTab(1) == "scilab.jhm"
    i = grep(jhmTab, "scilab.jhm")
    jhmTab(i) = []
    if grep(options, "x")==[] then
        jhmTab = ["scilab.jhm" ; jhmTab]
    else
        jhmTab($+1) = "scilab.jhm"
    end
    // scanning jhm files
    htmlName = ""

    if casse=="-C"
        grepCase = "i"
    else
        grepCase = ""
    end
    for jhm = jhmTab'
        //doc = xmlRead(jhm)
        //e = xmlXPath(doc,"//mapID[@target="""+fun+"""]")
       doc = mgetl(mapDir+jhm)
       i = grep(doc, "/target="""+fun+"""/"+grepCase, "r")
       if length(i)>1
           i = grep(doc, "/target="""+fun+"""/", "r")
       end
       if i~=[]
           search = "/^.*?target="""+fun+""" url=""(.+?.html).*?"".*$/"+grepCase
           htmlName = uman_strsubst(doc(i), search, "$1","r")
           module = strtok(jhm,".")
           break
       end
    end
    inATOMS = (find(atoms(:,1)==module)~=[])

   // item found in the doc
   // 
   if htmlName~="" then
       // in which jar file ?
       if module=="scilab"
           epath = pathconvert("\modules\helptools\jar\scilab_££_help.jar",%f)
       else
           epath = fs + module + fs + ".*?jar"+fs+"scilab_££_help.jar"
       end
       epath = strsubst(epath, "\", "\\")
       i = grep(jarList, "|^.*?"+strsubst(epath,"££",LANG)+"|", "r")
       if i==[]
           i = grep(jarList, "|"+strsubst(epath, "££", "en_US")+"|", "r")
           subDir = subDirEN
       else
           subDir = subDirLANG
       end
       jar = jarList(i(1))

       // is the HTML of this jar unzipped ?
       zippedFile = subDir + fs + htmlName
        if module=="scilab" 
            outDir = manDir         // a un / final
        else  // ATOMS and other contribs are mixed
            outDir = manAtomsDir    // idem
        end
        unzipped = outDir + subDir + fs + htmlName
       if isfile(unzipped) & grep(options, "r")==[]
           unzippedFile = unzipped
       else
           if isfile(unzipped) 
               deletefile(unzipped)
           end
           // We extract the HTML file:
           command = unzip + jar + " " + zippedFile + " -d " + outDir
           status = host(command)
           if status==0 // file found and extracted
               unzippedFile = unzipped
           end
       end
    else // item not found in the doc
         // =========================
        if grep(options, "x")==[] & ~isdef("uman_already_selfcalled", "n") then
            fun  = redirect_from_external(fun, options)
            // Some equivalences have "£" inserted: prefixing chars are options to force
            //  By default, "s" is forced, meaning that the equivalence is designed
            //  to target a parent summary:
            if grep(fun,"£")~=[] then
                s = strtok(fun, "£")
                fun = part(strchr(fun,"£"), 2:$)
                if s==fun
                    if grep(options, "s")==[]
                        options = "s" + options
                    end
                else
                    options = s + options
                end
            end
            // option "s" (summary) kills option "b" (bug)
            if grep(options, "s")~=[] then
                options = strsubst(options, "b", "")
            end
            bugOnWeb = grep(options, "w")~=[] & grep(options,"b")~=[]
            uman_already_selfcalled = %t
            uman(fun, options+"L"+LANG)
        end
    end

    // bugs report places
    // ------------------
    if module=="scilab" | htmlName=="" then
        // for Scilab modules
        BugzillaSearch = "http://bugzilla.scilab.org/buglist.cgi"+..
              "?query_format=advanced"+..
              "&o1=regexp&f1=short_desc&v1=(^|[^a-zA-Z])£[^0-9a-zA-Z%3D%3B]"+..
              "&bug_status=NEW&bug_status=ASSIGNED&bug_status=REOPENED"+..
              "&bug_status=RESOLVED&short_desc_type=allwordssubstr"+..
              "&order=resolution,bug_id DESC"
            // £ will be replaced with the item name
    else
        // for ATOMS modules
        if find(atoms(:,1)==module)~=[] then
            tmp = get_atoms_forge()
            k = find(tmp(:,1)==module)
            forgeName = tmp(k,2)
            if k ~= [] & forgeName~=""
                BugReport = "http://forge.scilab.org/index.php/p/"+forgeName+"/issues/"
                BugzillaSearch = BugReport + "search/?q="
                if unzippedFile~=""
                    BugzillaSearch = BugzillaSearch + fun
                else
                    BugzillaSearch = BugzillaSearch + fun0
                end
            else
                website = tmp(k,3)
                if website~=""
                    website = "http://atoms.scilab.org/toolboxes/"+module+"/#comments_remove"
                end
                BugReport = website
                BugzillaSearch = website
            end
        // for Contrib modules
        else
            BugReport = ""
            BugzillaSearch = ""
        end
    end
    // ---------------------------------------------------------------------

    // Display
    // =======
    msg1 = ""
    // Bugzilla | ticket/forge | website
    // ---------------------------------
    if bugOnWeb & BugzillaSearch~=""  then
        fun = strsubst(fun, "/^percent/", "%", "r")
        //symbols = "><[]:<>~=<=>==&|$.**-+././.^.*.\.'';"
        fun = uman_strsubst(fun, ..
            "/"+["&"   "\*"  "\."  "\-"  "\:"  "\]"  "\["  "\("  "\)"  "\$"  "\^"  "\|" ]+"/", ..
                ["%26" "\\*" "\\." "\\-" "\\:" "\\]" "\\[" "\\(" "\\)" "\\$" "\\^" "\\|"], "r")
        url = strsubst(BugzillaSearch,"£",fun)
        msg = _("uman","s: ""w"" option is not supported under ""%s""\n")
        if ~isdef("uman_already_selfcalled","n")
            uman_openURL(url, msg)
            if grep(options, ["s" "g"])==[]
                return
            end
        end
        return
    end
    //
    // Setting the flag allowing to target the console
    // ...............................................
    //  - whatever is the "g" status (can duplicate the output)
    //  - whether "w" is not used, OR (NOT "wb" AND NOT Scilab module)
    printConsole = (grep(options, "w")==[] | (~bugOnWeb & module~="scilab" ))
             
    if printConsole & grep(options,"j")~=[] 
        diary([], "pause") // http://bugzilla.scilab.org/show_bug.cgi?id=12551
    end
    // The required help file exists...
    // --------------------------------
    if unzippedFile~=""       // we go on on processing
        r = uman_grab(unzippedFile, fun, LANG)
        r.fun = fun0
        
        // targetting helpbrowser :
        // ......................
        global %helps
        if grep(options,"g")~=[] then 
            if module=="scilab" ..
                  | ( inATOMS & atomsIsLoaded(module)) ..
                  | ( ~inATOMS & find(%helps(:,2)==module)~=[] )
                // call the help browser in the chosen language
                if grep(options, "s")~=[]
                    tmp = r.parentSummary
                else
                    tmp = fun
                end
                helpbrowser(%helps(:,1), tmp, LANG, %f)
            else // we prepare a message
                if module~="scilab"
                    msg1 = _("uman", "module NOT LOADED => UNVAILABLE in HELP BROWSER")
                end
            end
        end

        // Targetting the console
        // ......................
        msg2 = ""
        if  printConsole
            if grep(options,"c")~=[] then
                clc()
            else    // separating zone
                tmp = lines()
                write(%io(2), part("=", ones(1,tmp(1))))
            end
            // item's page
            // - - - - - -
            msg0 = ""
            if grep(options, "s")==[]
                if module~="scilab"
                    if inATOMS
                        r.bug_report = BugReport
                        r.bugs_known = BugzillaSearch
                    else
                        r.bug_report = ""
                        r.bugs_known = ""
                    end
                end
                
                uman_disp(r, options)
                
                if grep(options,"w")~=[] & module~="scilab"
                    msg0 = " uman " + fun0 + " " + options0 + " :";
                    msg2 = " ["+module+"] "+..
                           _("uman","module EXTERNAL   => ONLINE help page UNAVAILABLE")
                end
                if msg1~="", msg1 = " [" + module + "] " + msg1, end
                tmp = part("-~", ones(1,max(length([msg1 msg2]))/2).*.[1 2])
                if tmp~=""
                    write(%io(2),tmp)
                    if (msg0~=""), write(%io(2), msg0), end
                    if (msg1~=""), write(%io(2), msg1), end
                    if (msg2~=""), write(%io(2), msg2), end
                    write(%io(2), " "+tmp)
                end
                
            // Parent summary
            // - - - -- - - -
            else 
                tmp = strsplit(unzippedFile, fs)
                zippedFile = tmp($-1) + fs + r.parentSummary + ".html"
                tmp = strsplit(unzippedFile, fs)
                outDir = strcat(tmp(1:$-2), fs)
                unzippedFile = dirname(unzippedFile) + fs + r.parentSummary + ".html"

                if fileinfo(unzippedFile)==[]
                    command = unzip + jar + " " + zippedFile + " -d " + outDir
                    status = host(command)  // status==0  <=> OK
                end
                
                // Parsing the summary page +++++++++++++++++++++++++++++
                html = mgetl(unzippedFile)                
                search  = [ "<html>" 
                            "<body>" 
                            "<head>.*?</head>" 
                            "<div class=""manualnavbar"">.*?</div>" 
                            "</body>" 
                            "</html>"
                            "<br />"
                            ]
                search = "|" + search + "|"
                replace = ["<div>" "" "" "" "" "</div>" ascii(10) ]
                html2 = uman_strsubst(strcat(html, " "), search, replace, "r")
                tmp = xmlReadStr(html2)
                simpleContent = %t
                text = uman_xml2txt(tmp.root, " ")
                tmp2 = strsplit(text(1), ">")
                if grep(tmp2(1),"Scilab")~=[] then 
                    tmp2(1) = "Scilab"
                end
                tmp2 = stripblanks(tmp2)
                tmp2(find(tmp2=="")) = []
                if tmp2(1)==tmp2(2)
                    tmp2(2) = []
                end
                text(1) = strcat(tmp2," > ")
                text(2) = []
                // Displaying the summary page ++++++++++++++++++++++++++
                write(%io(2), text)
                xmlDelete(tmp)
            end
            // targeting the console
        end   
        
        // Targetting ONLINE pages
        // -----------------------
        printWeb = ( grep(options,"w")~=[] & grep(options,"b")==[] & ..
                     grep(options,"s")==[] &  module=="scilab") 
        if printWeb & ~uman_alreadyWebed then
            url = "http://help.scilab.org/docs/current/"
            supported_online= ["en_US" "fr_FR" "pt_BR" "ru_RU" "ja_JP"]
            if and(LANG~=supported_online)
                langH = "en_US"
            else
                langH = LANG
            end
            url = url+langH+"/"+fun+".html"
            msg = _("uman", "%s: ""w"" option is not supported under ""%s""\n")
            msg = msprintf(msg, "uman", getos())
            uman_openURL(url, msg)
            uman_alreadyWebed = %t
        end
        
    // The required help file DOES NOT EXIST
    // -------------------------------------
    else                   
        if grep(options, "w")==[]
            // we display the heading comments (if any)
            ierr = execstr("Fun="+fun0+";","errcatch")
            if ierr==0 & typeof(Fun)=="function" then
                if grep(options,"c")~=[] then
                    clc()
                end
                if ~isdef("uman_HeadCommented", "n")
                    head_comments(fun0)
                    uman_HeadCommented = %t
                end
            elseif isdef("uman_already_selfcalled","n") | grep(options, "x")~=[]
                msg = _("uman","%s: ""%s"" is not documented\n")
                disp(msprintf(msg,"uman",fun0))
            end
        elseif ~uman_alreadyWebed & grep(options, "s")==[] // search on online help
            url = "http://help.scilab.org/docs/current/"
            supported_online= ["en_US" "fr_FR" "pt_BR" "ru_RU" "ja_JP"]
            if and(LANG~=supported_online)
                langH = "en_US"
            else
                langH = LANG
            end
            url = url + langH + "/search/" + fun0
            msg = _("uman","%s: ""w"" option is not supported under ""%s""\n")
            msg = msprintf(msg, "uman", getos())
            uman_openURL(url, msg)
        end
    end
    if printConsole & grep(options,"j")~=[] 
        diary([], "resume") // http://bugzilla.scilab.org/show_bug.cgi?id=12551
    end
endfunction

// ===========================================================================

// Copyright (C) - 2015 - Samuel GOUGEON
// Utility function for uman()

function uman_disp(man_contents, options)
    // Finally displays the formatted help text content into the console
    if or(part(LANG,[1 2])==["ja" "zh"])
        lengthFactor = 1.5
    else
        lengthFactor = 1
    end
    u = "----------------------------------------------------------"+..
        "----------------------------------------------------------"
    s = man_contents
    c = %io(2)
    
    t = s.path
    write(c, ["" ; t ; part(".",ones(1,length(t))) ])

    //t = "uman "+s.fun
    //write(c, [ t ; part(u,1:length(t)*lengthFactor) ])
    write(c, ["" ; strcat(justify([s.refname s.refpurpose],"l")," - ","c")])
    
    if s.synopsis~=[] then
        t = s.synopsisTitle
        write(c, [ "" ; t ; part(u,1:length(t)*lengthFactor) ; " "+s.synopsis ])
    end
    // Loop over other sections, in the order they appear in the original page
    sections = man_contents.sections
    ns = size(sections,"*")
    for is = 1:ns
        s = sections(is)
        tmp = s.type
        if or(tmp==["parameters" "see_also"]) | ..
            (grep(options,"d")~=[] & tmp=="description") | ..
            (grep(options,"e")~=[] & tmp=="examples") | ..
            (grep(options,"h")~=[] & tmp=="history")
            t = s.title
            write(c, [ "" ; t ; part(u,1:length(t)*lengthFactor)])
            if tmp=="see_also"
                s.contents = part(s.contents, 3:$)
            end
            write(c, " "+s.contents)
        end
    end
    // Bugs
    if grep(options,"b")~=[] 
        t = _("uman","Bugs")
        write(c, [ "" ; t ; part(u,1:length(t)*lengthFactor) ])
        write(c, [ _("uman","Known")+"  : " + man_contents.bugs_known ; ..
                   _("uman","Report")+" : " + man_contents.bug_report ])
    end
endfunction

// ===========================================================================

// Copyright (C) - 2015 - Samuel GOUGEON
// Utility function for uman()

function r = uman_grab(htmlf, fun, LANG, options)
    // htmlf : HTML file name
    // fun : function's name or item
    // LANG: ##_## language string
    // options: options string from uman()
    // r. : structure with fields
    //  .fun .title  .path .refname .refpurpose .synopsisTitle .synopsis
    //  .paramTitle  .parameters .examples .seeAlsoTitle .seeAlso 
    //  .bug_report  .bugs_known
    //  .parentSummary

    lang = part(LANG,1:2)
    
    html = mgetl(htmlf)

    search  = ["/<br>/" "/<meta .*?>/" ]
    replace = ["<br/>"     ""      ]
    html2 = uman_strsubst(strcat(html, " "), search, replace, "r")
    
    // URLs GET's & separators must be replaced with %26 in order to not break
    //  XML parsing. Replacing all cannot be done with TCL uman_strsubst because
    //  "&" has a special meaning in this TCL regsub context 
    // => using a while loop here: 
    src = ""
    while or(html2~=src) do
        src = html2
        html2 = uman_strsubst(src, "|(href=""http:[^\""]*?)&([^\""]*?"")|", "$1%26$2", "r")
    end
    
    doc = xmlReadStr(html2)

    // Table of <pre.. </pre> sections (badly processed as XML)
    [s,e,preTable] = regexp(strcat(html,"££"),"%<pre[\s>].*?</pre>%mi")
    e = xmlXPath(doc,"//pre")
    for i = 1:size(s,"*")
        tmp = preTable(i)
        preTable(i) = uman_cleantxt(preTable(i),"tags|htmlent|style")
        // now we forge an id = full content cleaned
        preTableNoTag(i) = uman_strsubst(e(i).content, [" " "££"], ["" ""]) 
    end

    htmlcat = strcat(html, "££")
    
    r.fun = fun
    
    // Title
    e = xmlXPath(doc,"//title")
    r.title = e.content

    // Path in help
    // ------------
    e = xmlXPath(doc,"//span[@class=""path""]")
    tmp = strsplit(e.content, ">")
    if grep(tmp(1), "Scilab")~=[] then 
        tmp(1) = "Scilab"
    end
    tmp = stripblanks(tmp)
    tmp(find(tmp=="")) = []
    if tmp(1)==tmp(2)
        tmp(2) = []
    end
    r.path = strcat(tmp," > ")
    
    // Parent Summary (from path)
    tmp = "%<span class=""path"">.*?</span>%mi"
    [s, e, tmp] = regexp(html2, tmp)
    tmp2 = "%href="".*?""%mi"
    [s, e, tmp2] = regexp(tmp, tmp2)
    r.parentSummary = part(tmp2($), 7:$-6)
    
    // URL(report a bug)
    tmp = "%""http://bugzilla\.scilab\.org/enter_bug\.cgi.*?\""%"
    [s,e,m] = regexp(htmlcat, tmp)
    if m~="" then
        r.bug_report = strsubst(m,"""","")+"&short_desc="+fun
    else
        r.bug_report = "http://bugzilla.scilab.org/enter_bug.cgi"+..
        "?product=Scilab%20software&component=Documentation%20pages"+..
        "&short_desc="+fun
    end
    // URL(known bugs)
    // BugzillaSearch is defined in the calling level
    r.bugs_known = strsubst(BugzillaSearch,"£",fun)
    
    // refname(s)
    e = xmlXPath(doc,"//h1[@class=""refname""]")
    r.refname = xmlAsText(e)'
    
    // refpurpose
    e = xmlXPath(doc,"//p[@class=""refpurpose""]")
    r.refpurpose = uman_cleantxt(xmlAsText(e)', "blanks")
        // Some pages presenting multiple equivalent functions do not repeat
        // the short description for last aliases 
        // (printf.. http://bugzilla.scilab.org/13748)
        // We must complete them here:
    tmp =  r.refpurpose
    if size(tmp,1)<size(r.refname,1) then
        r.refpurpose = [tmp ; emptystr(size(r.refname,1)-size(tmp,1),1)+r.refpurpose($)]
    end
    
    // synopsis title
    CS = struct("en","Calling Sequence",..
                "fr","Usages",..
                "ja","呼出し手順", ..
                "pt","Seqüência de Chamamento",..
                "de","Aufruf", ..
                "ru","Последовательность вызова",..
                "zh","调用序列")
    if isfield(CS,lang) then
        r.synopsisTitle = CS(lang)
    else
        e = xmlXPath(doc,"//div[@class=""refsynopsisdiv""]")
        if e.size>0 then
            r.synopsisTitle = _(e(1).children(1).content)
        else
            r.synopsisTitle = "Calling Sequence"
        end
    end

    // Synopsis
    //s = xmlXPath(doc,"//div[@class=""synopsis""]")
    [s,e,M] = regexp(htmlcat, "%<div class=""synopsis""><pre>(.*?)</pre>%")
    tmp = []
    for m = M'
        s = strsplit(uman_cleantxt(m,"tags|htmlent|style"),"££")
        tmp = [tmp ; s ]
        if size(s,1)>1 then
            tmp = [tmp ; ""]
        end
    end
    if tmp($)=="" then // removing the last blank line
        tmp($) = []
    end
    if size(r.refname,1)==1 then    // Removing blank lines
        tmp(find(tmp=="")) = []
    end
    r.synopsis = tmp
    
    // REFSECTIONS
    // ===========
    refsections = xmlXPath(doc,"//div[@class=""refsection""]")
    nrs = length(refsections)

    // MAIN LOOP over SECTIONS of all TYPES
    // ------------------------------------
    for i = 1:nrs
        rs = refsections(i)
        secTitle = xmlXPath(rs, "./h3[@class=""title""]")
        if size(secTitle,"*")>0
            tmp = uman_cleantxt(secTitle(1).content, "blanks")
            [secType, typeName] = typeof_section(tmp, lang)
            r.sections(i).title = typeName
            if secType=="examples"
            end
            
            contents = uman_xml2txt(rs, "")
            contents(1) = []     // suppressing the embedded title
        else
            secType = "description"
            r.sections(i).title = ""
            contents = uman_xml2txt(rs, "")
        end
        r.sections(i).type  = secType
        if secType=="examples" then
            contents = uman_strsubst(contents, "/\[IMAGE\: .*?\]/","","r")
            
        elseif secType=="see_also"
            k = find(stripblanks(contents)=="")
            contents(k) = []
        end
        
        r.sections(i).contents = contents
    end

    //  History : de = Historische
    xmlDelete(doc)
endfunction

// ==========================================================================

// Copyright (C) - 2015 - Samuel GOUGEON
// Utility function for uman()

function [secType, typeName] = typeof_section(secTitle, lang)
    // id vs titles for sections    
    langs = [ "en" "de" "fr" "ja" "pt" "ru" "zh" ]
    ID = ["parameters"              "description"  "examples" ]
    en = ["Parameters,Parameter,arguments"    "Description"  "Examples,example"]
    de = ["Parameter"               "Beschreibung" "Beispielen,beispiel,beispiele"]
    fr = ["Paramètres"              "Description"  "Exemples,exemple"]
    ja = ["設定,引数,パラメータ"       "説明"         "例"]
    pt = ["Parâmetros,Argumentos"   "Descrição"    "Exemplos,exemplo"]
    ru = ["Аргументы"               "Описание"     "Примеры,Примерами,ример,примеров"]
    zh = ["設置"                    "描述"          "示例,例子"]

    ID = [ ID "see_also"    "history"   ]
    en = [ en "See Also"    "History"   ]
    de = [ de "Siehe auch"  "Historische"]
    fr = [ fr "Voir aussi"  "Historique"]
    ja = [ ja "参照"         "履歴"      ]
    pt = [ pt "Ver também"  "Histórico" ]
    ru = [ ru "Смотрите также" "История"]
    zh = [ zh "参见"         "历史的"    ]

    k = find(langs==lang)
    if k==[] then
        lang = "en"
    end
    execstr("Ref = "+lang)
    secType = "description" // default
    typeName = secTitle     // default
    secTitle= convstr(secTitle)
    
    ref = convstr(Ref)
    found = %f
    while ~found
        for i=1:size(ref, 2)
            tmp = strsplit(ref(i), ",") 
            j = grep(secTitle, tmp)
            if j~=[] then
                secType = ID(i)
                if and(i~=[2 3])
                    typeName = strtok(Ref(i), ",")
                end
                found = %t
                break
            end
        end
        tmp = convstr(en)
        if found | and(ref==tmp), break, end
        ref = tmp
    end
endfunction
// ==========================================================================

// Copyright (C) - 2015 - Samuel GOUGEON
// Utility function for uman()
function jarList = uman_listJar()
    // Returns de column matrix of paths to ALL JAR files (not only for LANG or en_US)
    
    function flist = dirjar(path)
        // path must be ended with filesep()
        if isdir(path) then
            if getos()=="Windows" then
                flist = unix_g("dir /s /b "+path+"scilab_*_help.jar 2> nul")
            elseif getos()=="Linux"
                flist = unix_g("find "+path+" -name scilab_*_help.jar 2> /dev/null")
            else
                flist = unix_g("find 2>/dev/null "+path+ "scilab_*_help.jar")
            end
            if flist=="" then
                flist = []
            end
        else
            flist = []
        end
    endfunction
    
    jarList = []
    // Scilab trunk:
    path = pathconvert(pathconvert(SCI)+"modules\helptools\jar\")
    jarList = [jarList ; dirjar(path)]
    // Contrib (ATOMS and others) : ALL USERS
    path = pathconvert(pathconvert(SCI)+"contrib\")
    jarList = [jarList ; dirjar(path)]
    // USER : ATOMS
    path = pathconvert(pathconvert(SCIHOME)+"atoms\")
    jarList = [jarList ; dirjar(path)]
    // USER : other contribs
    path = pathconvert(pathconvert(SCIHOME)+"contrib\")
    jarList = [jarList ; dirjar(path)]
endfunction

// --------------------------------------------------------------------------

// Copyright (C) - 2015 - Samuel GOUGEON
// Utility function for uman()
function extractJhelpMaps(jarList, atoms, unzip, manDir, mapDir)
    // From the list of selected JAR files, for each file, extracts its 
    //  jhelpmap.jhm in a common directory, and renames it according to the
    //  module
    fs = filesep()
    pattern = pathconvert("\modules\helptools\jar\scilab_", %f)
    for jar = jarList'
        contrib? = %f
        if grep(jar, pattern)~=[]
            module = "scilab"
        else
            if atoms~=[]
                [tmp, ind ] = grep(getshortpathname(jar), atoms(:,4))
            else
                ind = []
            end
            if ind~=[] 
                module = atoms(ind(1),1)    // Name = id in ATOMS
            else
                // The name of the module (such as stored in %helps(:,2))
                //  can be retrieved through the file name
                //  ~\etc\modulename.start
                tmp = pathconvert(dirname(jar)+fs+"..\etc\*.start", %f)
                if getos()=="Windows" then
                     startfile = unix_g("dir /b " + tmp + " 2> nul")
                elseif getos()=="Linux"
                    startfile = unix_g("ls -1 " + tmp + " 2> /dev/null")
                else
                    tmp = pathconvert(dirname(jar)+"\..\etc *.start", %f)
                    startfile = unix_g("find 2>/dev/null "+tmp)
                end
                module = basename(startfile)
                contrib = %t
            end
        end
        lang = uman_strsubst(jar, "|.*?_([a-z]{2}_[A-Z]{2})_help.jar|", "$1", "r")
        subDir = "scilab_"+lang+"_help"+fs
        zippedMap = subDir + "jhelpmap.jhm"
        command = unzip + jar + " " + zippedMap + " -d "+manDir
        status = host(command)
        unzippedMapFrom = manDir + subDir + "jhelpmap.jhm"
        unzippedMapTo   = mapDir + module + ".jhm" 
        [s, m] = movefile(unzippedMapFrom, unzippedMapTo)
    end
endfunction

// --------------------------------------------------------------------------

// Copyright (C) - 2015 - Samuel GOUGEON
// Utility function for uman()
function out = uman_cleantxt(txt, cleanjobs)
    // cleanjobs: "tags|htmlent|catlines|style|blanks"
    cleanjobs = strsplit(cleanjobs,"|")'
    out = txt
    for cleanjob = cleanjobs
        select cleanjob
        case "tags"
             out = uman_strsubst(out, "/<.*?>/", "", "r")
         case "catlines"
             for i = 1:size(out,"*") 
                out(i) = strcat(stripblanks(strsplit(out(i),"££"))," ")
             end
         case "htmlent"
             patterns = ["&#0034;" "&#0038;" "&#0039;" "&#0060;" "&#0062;" "&gt;" "&lt;" "&quot;" ]
             replaces = [ """"       "\&"       "''"      "<"       ">"       ">"   "<"   """"    ]
            out =  uman_strsubst(out, patterns , replaces)
         case "style"
            out =  uman_strsubst(out, "/([^ \=<>~])=([^ \=])/", "$1 = $2", "r")
            out =  uman_strsubst(out, "/([^\s]);([^\s])/", "$1; $2", "r")
        case "blanks"
            out = uman_strsubst(out, "/(\s{2,})/", " ", "r")
        end
    end
endfunction

// --------------------------------------------------------------------------

// Copyright (C) - 2015 - Samuel GOUGEON
// Utility function for uman()

function out = uman_wraplines(text, ncol)
    // LANG must be set in the calling environnement
    if or(LANG==["ja_JP" "zh_CN"]) then
        nchar = floor(ncol/1.5)  // because symbols are wider than latin ones
    else
        nchar = ncol
    end
    nchar = nchar-1
    text = stripblanks(text)
    Sep = " "
    if LANG=="ja_JP" then
        Sep = [Sep "。"]
    end
    [T, sep] = strsplit(text, Sep)
    sep($+1) = ""
    LT = length(T)+1
    out = []
    line = ""   // current line
    lineL = 0   // lineLength
    for i = 1:size(T,"*")
         if lineL+LT(i)<=nchar | line==""
             line = line + T(i) + sep(i)
             lineL = lineL + LT(i)
         else
             out = [out ; line]
             line = T(i) + sep(i)
             lineL = LT(i)
         end
    end
    if stripblanks(line,%t)~="" then
        out = [out ; line]
    end
    out = stripblanks(strsubst(out, "¤"," "))
endfunction

// --------------------------------------------------------------------------

// Copyright (C) - 2015 - Samuel GOUGEON
// Utility function for uman()

function [txt, inline] = uman_xml2txt(node, lineHead)
    
    // <dl> : definitions list : ignorer content ; traiter les children
    //   <dt><span class="term">..</span>      : definition term
    //       <dd><p class="para">...</p></dd>  : definition definition
    //   </dt>
    // </dl>
    //<code..>bla bla</code>   : =>  [bla bla]  // cancelled
    //<a href=""..>bla bla</a> : => _{bla bla}  // cancelled
    // <ul class="itemizedlist">
    //       <li><p class="para">..</p> : => nouvelle ligne à chaque <p..>
    //           <p..>...</p>
    //       </li>
    // </ul>
    // <img ..> among which 
    //   <img src="ScilabNote.png"/> and <img src="ScilabWarning.png"/>
    //   <img src="ScilabTip.png"/> and <img src="ScilabCaution.png"/>

    inline = %F
    thistxt = []
    thisline = []
    name = convstr(node.name)
    nc = node.children.size     // number of children
    // ----------------------------------------------------------------
    if name=="code" then
        txt = strsubst(uman_cleantxt(node.content, "style|blanks"), " ", "¤")
        inline = %T
        
    elseif or(name==["text" "a" "i"]) then
        txt = uman_cleantxt(node.content, "blanks")
        inline = %T
        
    // ----------------------------------------------------------------
    elseif or(name==["b" "em"]) then
        txt = uman_cleantxt(node.content,"blanks")
        if  name=="em" then
            txt = "/"+txt+"/"
        else
            txt = convstr(txt,"u")
        end
        inline = %T
        
    // ----------------------------------------------------------------
    elseif name=="varname"
        txt = "[" + uman_cleantxt(node.content, "blanks") + "]"
        inline = %T

    // ----------------------------------------------------------------
    //    elseif name=="a" 
    //        // <a> over complex content will ignore content's format :
    //        //txt = "_{" + node.content + "}"
    //        txt = node.content
    //        inline = %T

    // ----------------------------------------------------------------
    elseif or(name==["h3" "div" "p" "span" "li" "dd"])
        if name=="div" & node.attributes.class~=[] & node.attributes.class=="programlisting" 
            k = find(preTableNoTag==strsubst(node.content," ",""))
            if k~=[]
                txt = strsplit(preTable(k(1)),"££")
            else
                txt = ""
            end
        else
            tmp = lines()
//            wW = tmp(1)-length(lineHead)-3    // Wrapping width
            wW = tmp(1)-length(lineHead)    // Wrapping width
            if wW<30, wW = 30, end            // minimal width = 30 chars
            forceInLine = (name=="li" & isdef("simpleContent", "n"))
            if name=="li" & forceInLine
                tmp = xmlXPath(node, "./ul")  // are there any nested sub-list?
                if size(tmp, "*")>0         //   yes:
                    forceInLine = %f
                end
            end

            for i = 1:nc
                [ tmp, inL ] =  uman_xml2txt(node.children(i), lineHead)
                if inL | forceInLine then
                    thisline = thisline + strcat(tmp, " ")
                else 
                    tmp2 = uman_wraplines(uman_cleantxt(thisline,"blanks"), wW)
                    thistxt = [ thistxt ; tmp2 ; tmp ]
                    thisline = []
                end
            end
            if thisline~=[]
                tmp2 = uman_wraplines(uman_cleantxt(thisline,"blanks"), wW)
                txt = [ thistxt ; tmp2 ]
            else
                txt = thistxt
            end
            if name=="span" & size(txt,1)==1 then
                inline = %T
            end
        end
        
    // ----------------------------------------------------------------
    elseif name=="pre"
        // preTable, preTableNoTag = external variables 
        k = find(preTableNoTag==strsubst(node.content, " ", ""))
        txt = strsplit(preTable(k(1)), "££")

    // ----------------------------------------------------------------
    elseif name=="img"  // ex: <img src='jarsci:./PlotSparse_1.png'/>
        // works with Scilab pages. Untested with external packages
        tmp = node.attributes.src;
        tmp = tokens(tmp,"/")
        txt = "[IMAGE: "+tmp($)+"]"
        inline = %t

    // ----------------------------------------------------------------
    elseif name=="dl"     // definitions list
        // Processing all <dt> entries
        terms = []     // matrix of terms
        defs  = cell() // cell of related definitions
        i = 1    // n° of <dl>'s children
        it = 0   // n° of terms
        maxL = 8   // maximal length of "alignable" terms
        minL = 5    // minimal indentation of lines for the bloc
        while i<=nc
            // only <dt>..</dt> should be considered:
            while node.children(i).name~="dt" & i<nc
                i = i+1
            end
            dt = node.children(i)
            sp = dt.children(1) // should be <span class="term">..</span>
            it = it+1
            terms(it) = stripblanks(strtok(sp.content, ":"))
            // looking for the forthcoming <dd> section:
            sdt = dt.children.size
            j = min([2,sdt])   // n° of <dt>'s children
            while j<sdt & convstr(dt.children(j).name)~="dd" 
                j = j+1
            end
            //dd = dt.children(j) should be <dd><p class="para">...</p></dd>
            // local header length (estimated)
            k = find(length(terms)<maxL)
            if k~=[]
                hL = max(length(terms(k)))+3
            else
                hL = minL
            end
            ddtxt = uman_xml2txt(dt.children(j), lineHead+part(" ",ones(1:hL)))
            defs(it).entries = ddtxt
            // nothing is expected between </dd> and the embedding </dt>
            i = i+1
        end

        // Building the whole set of <dt> entries
        // --------------------------------------
        // For short terms, the description is appended to term's line :
        // Setting the standard width of items label field
        I = find(length(terms)<maxL)
        if I~=[] then
            nC = max(length(terms(I))) + 3  // +3 for " : "
            terms(I) = justify(terms(I),"l")
        else
            nC = minL
        end
        // Setting the local blank header
        header = part(" ", ones(1:nC))
        
        // Setting the standard width of item Description field
        tmp = lines()
        wD = max([30, tmp(1)-length(lineHead+header) ])
        
        // Loop on items
        for L = 1:size(terms,1)
            wD_line1 = wD - max([ 0 (length(terms(L))+3-nC)])
            tmp = defs(L).entries
            if size(tmp,1)<2 then
                tmp2 = uman_wraplines(tmp, wD_line1+1)
                thistxt =[thistxt; terms(L)+" : "+tmp2(1)]
                if tmp~=tmp2(1) then
                    tmp2 = uman_wraplines(strcat(tmp2(2:$)," "), wD)
                    thistxt  = [thistxt; ""; header + tmp2]
                end
            else
                if wD_line1 < wD
                    // The first line is kept clear
                    thistxt =[thistxt; terms(L)+" : " ;  header+tmp ]
                else
                    thistxt =[ thistxt ; terms(L)+" : "+tmp(1); header + tmp(2:$)]
                end
            end
            if size(tmp,1)>1 & stripblanks(thistxt($))~="" then
                thistxt($+1) = ""
            end
        end
        txt = " "+thistxt
    // ----------------------------------------------------------------
    elseif or(name==["ul" "ol"])
        // Processing all <li> entries :
        litab = []
        for i = 1:nc
            li = node.children(i)
            tmp = uman_xml2txt(li, "  "+lineHead)
            if name=="ul"
                tmp(1) = "* "+tmp(1)
            else
                tmp(1) = sprintf("%2d",i)+". "+tmp(1)
            end
            if size(tmp,1)>2 & stripblanks(tmp($))~="" then
                tmp($+1) = ""
            end
            tmp(2:$) = "  "+tmp(2:$)
            litab = [ litab ; tmp ]
        end
        if size(litab,1)>8 then
            litab = [""; litab]
        end
        txt = " "+litab
        
        // Post-processing for the list of summaries, in order to align the "--"
        if grep(options, "s")~=[]
            K = grep(txt, "/^ \*.*?—/","r")
            Kw = grep(txt, "/^   [^\s\*]/", "r")  // wrapped line
            nL = size(txt,1)
            if K~=[] then
                txt1 = stripblanks(txt(K))
                s = strcspn(txt1,"—")            // locating the "—" separator
                ms = mean(s)
                ss = stdev(s)
                p = min([18, round(ms+ss)])
                j = find(s>0 & s<p)       // # of lignable lines
                t = []
                for m = j
                    t = [ t ; strsplit(txt1(m),s(m))']
                end
                p = min([18, max(length(t(:,1)))+1])
                t = justify(t(:,1), "l") + " " + t(:,2)
                txt1(j) = t
                txt(K) = txt1
                // Shifting wrapped lines
                head = part(" ", ones(1,p+3))
                cw = lines()                             // Console's width
                ww = max([30, cw(1)-length(lineHead)-p]) // wrapping width
                
                newtxt = []
                line = ""
                for i = 1:size(txt,"*")
                    if or(i==Kw) 
                        line = line + " "+ stripblanks(txt(i))
                    else
                        if line~=""
                            // we process the registered line
                            tmp = uman_wraplines(line, ww+p+3)
                            newtxt = [ newtxt ; " "+tmp(1) ]
                            if size(tmp,1)>1
                                tmp = uman_wraplines(strcat(tmp(2:$), " "), ww)
                                newtxt = [newtxt ; head + tmp ]
                            end
                            line = " "
                        end
                        if or(i==K)
                            line = " "+txt(i)
                        else
                            newtxt = [newtxt ; txt(i)]
                        end
                    end
                end
                if line~=" "
                    // we process the registered line
                    tmp = uman_wraplines(line, ww+p+3)
                    newtxt = [ newtxt ; " "+tmp(1) ]
                    if size(tmp,1)>1
                        tmp = uman_wraplines(strcat(tmp(2:$), " "), ww)
                        newtxt = [newtxt ; head + tmp ]
                    end
                end
            end
            txt = newtxt
        end
//     ----------------------------------------------------------------
    elseif or(name==["table" "tbody"])
        // <note> and <warning> are in a table starting with an image:
        //   <table><tr><td valign="top"><img src="ScilabNote.png"/>
        tmp = node.children(1).children(1).children(1)
        if tmp.name=="img"
            if grep(tmp.attributes.src, ["ScilabNote.png" "ScilabTip.png"])~=[]
                txt = _("uman", "NOTE: ")
            elseif grep(tmp.attributes.src, ["ScilabWarning.png" "ScilabCaution.png"])~=[]
                txt = _("uman", "WARNING: ")
            else
                txt = ""
            end
            txt = txt + uman_cleantxt(node.content+" ","blanks")
            w = lines()
            txt = uman_wraplines(txt, max(30, w(1)-length(lineHead)))
            return
        end
        a = node.attributes
        TABLE = %t
        Array = []
        for i = 1:nc        // Loop over rows
            row = uman_xml2txt(node.children(i)) 
            if i>1 
                // missing cells?
                if size(row,2)< size(Array,2)
                    row = resize_matrix(row, 1, size(Array,2))
                elseif size(row,2)> size(Array,2)
                    Array = resize_matrix(Array, 1, size(row,2))
                end
            end
            Array = [Array ; row ]
        end
        // If there are empty (or blank) columns => we remove them
        L = length(stripblanks(strcat(Array,"","r")))
        Array(:, find(L==0)) = []
        
        Array = justify(Array, "l")
        if a.border~=[] & a.border~="0"
            Array = strcat(Array,"| ","c")
            Array = "| " + Array + " |"
            hline = part("-",ones(1,length(Array(1,:))))
            txt = " " + [hline ; Array ; hline ]
        else
            Array = strcat(Array," ","c")
            if size(Array,1)==1 then
                txt = stripblanks(Array)
            else
                txt = " " + Array
            end
            
        end
    // ----------------------------------------------------------------
    elseif name=="tr"
        Row = node.children
        row = []
        if nc>1
            // We assume that <td> cells have simple contents:
            for i = 1:nc
                row($+1) = uman_cleantxt(Row(i).content+" ","blanks")
            end
            txt = row.'
        else
            txt = uman_xml2txt(Row(1), lineHead)
        end
    // ----------------------------------------------------------------
    else 
        txt = "<"+name+">"+node.content+"</"+name+">"
    end
endfunction

// --------------------------------------------------------------------------

// Copyright (C) - 2015 - Samuel GOUGEON
// Utility function for uman()
function uman_openURL(url, msg)
    if getos()=="Windows"
        winopen(url)
    elseif getos()=="Linux"
        unix("xdg-open """+url+"""")
    elseif getos()=="Darwin"         // Mac OS
        unix("open """+url+"""")
    else
        warning(msg)
    end
endfunction

// --------------------------------------------------------------------------

// Copyright (C) - 2015 - Samuel GOUGEON
// Utility function for uman()
function rep = redirect_from_external(item, options)
// List of equivalences or nearest redirections
//  column #1: external item (Octave, etc...)
//  column #2: Scilab redirection (help's filename).
//             "£" prefix = options to be passed when calling uman
e = [ "all"     "and"
      "angle"   "atan"
      "any"     "or"
      "arrayfun" "feval"
      "atan2"   "atan"
      "axes"    "newaxes"
      "axis"    "gca"
      "balance" "balanc"
      "box"     "gca"
      "caxis"   "colorbar"
      "chebwin" "window"
      "cla"     "delete"
      "clear global"    "clearglobal"
      "colormap" "jetcolormap"
      "colors"   "d£color_list"       // inner redirection
      "compan"   "companion"
      "contourc" "contour2di"
      "contour3" "contour"
      "copyobj" "copy"
      "csvread" "csvRead"
      "datatype" "d£type"           // inner redirection with "d" forced
      "dbclear" "delbpt"
      "dbcont"  "resume"
      "dbquit"  "abort"
      "dbstack" "where"
      "dbstatus" "dispbpt"
      "dbstop"  "setbpt"
      "dbup"    "return"
      "deblank" "stripblanks"
      "demo"    "demo_gui"
      "dlmwrite" "fprintfMat"
      "echo"    "mode"
      "eig"     "eigs"
      "ellipj"  "amell"
      "ellipke" "percentk"
      "eps"     "percenteps"
      "errorbar" "errbar"
      "errordlg" "messagebox"
      "false"   "%f"
      "fclose"  "mclose"
      "feof"    "meof"
      "files"   "£listfiles"     // inner summary
      "fftn"    "fft"
      "fgets"   "mgetl"
      "fill"    "xfpolys"
      "flipud"  "flipdim"
      "fliplr"  "flipdim"
      "frewind" "mseek"
      "gallery" "testmatrix"
      "gammainc" "cdfgam"
      "ginput"  "locate"
      "graphics" "£xsetech"     // inner summary
      "grid"    "xgrid"
      "gtext"   "legends"
      "hdf5"    "£h5read"       // inner summary http://www.hdfgroup.org/HDF5/
      "helpdlg" "messagebox"
      "helpwin" "help"
      "hggroup" "glue"
      "hgload"  "load"
      "hgsave"  "save"
      "hist"    "histplot"
      "hold"    "axes_properties"
      "horzcat" "cat"
      "i"       "%i"
      "j"       "%i"
      "ifft2"   "ifft"
      "ifftn"   "ifft"
      "imagesc" "Matplot1"
      "inf"     "%inf"
      "invfeqz" "frfit"
      "iskeyword" "getscilabkeywords"
      "ismac"   "getos"
      "ismember" "members"
      "ispc"    "getos"
      "isunix"  "getos"
      "kaiser"  "window"
      "keyboard" "pause"
      "legend"  "legend_properties"
      "lenght"  "length"        // common mis-spelling
      "listdlg" "x_choose"
      "loglog"  "plot2d"
      "lookfor" "apropos"
      "lower"   "convstr"
      "magic"   "testmatrix"
      "matlabroot" "SCI"
      "metanet" "£bandwr"       // inner redirection: "uman metanet s" will be OK
      "mfilename"  "get_absolute_file_path"
      "mod"     "pmodulo"
      "more"    "lines"
      "nan"     "%nan"
      "nargin"  "argn"
      "nargout" "argn"
      "nonzeros" "spget"
      "num2str" "string"
      "ode23"   "ode"
      "ode45"   "ode"
      "ode113"  "ode"
      "ode15s"  "ode"
      "odeget"  "odeoptions"
      "paddarray"   "resize_matrix"
      "pi"      "%pi"
      "planerot" "givens"
      "plot3"   "param3d"
      "plottools" "ged"
      "polar"   "polarplot"
      "polyder" "derivat"
      "polyval" "horner"
      "printdlg" "toprint"
      "psi"     "dlgamma"
      "questdlg" "messagebox"
      "randi"   "grand"
      "randn"   "rand"
      "randperm" "grand"
      "rbbox"   "rubberbox"
      "realmax" "number_properties"
      "realmin" "number_properties"
      "regexprep"  "strsubst"
      "rem"     "modulo"
      "reshape" "matrix"
      "rmfield" "null"
      "rot90"   "flipdim"
      "rotate3d" "rotate_axes"
      "run"      "exec"
      "semilogx" "plot2d"
      "semilogy" "plot2d"
      "shg"      "show_window"
      "shortcuts" "d£console"     // inner redirection
      "signal"   "£detrend"     // inner summary
      "sort"     "gsort"
      "sortrows" "gsort"
      "sparses"  "£chsolve"     // inner summary
      "sprandn"  "sprand"
      "sprandsym" "sprand"
      "spy"      "PlotSparse"
      "stairs"   "plot2d2"
      "statistics" "£reglin"    // inner summary
      "stem"     "plot2d3"
      "strfind"  "strindex"
      "strjoin"  "strcat"
      "strjust"  "justify"
      "strtrim"  "stripblanks"
      "switch"   "select"
      "tempdir"  "TMPDIR"
      "texlabel" "math_rendering_features_in_graphic"
      "text"    "xstring"
      "textscan" "msprintf"
      "true"    "%t"
      "tukeywin" "window"
      "uisetcolor" "uigetcolor"
      "unix"    "unix_g"
      "upper"   "convstr"
      "variables" "£who"            // inner summary
      "version" "getversion"
      "vertcat" "cat"
      "view"    "axes_properties"
      "waitforbuttonpress" "xclick"
      "warndlg" "messagebox"
      "which"   "get_function_path"
      "windows"  "£winopen"          // inner summary
      "whos -file" "listvarinfile"
      "xml"     "£xmlRead"          // inner summary
      "xlim"    "axes_properties"
      "ylim"    "axes_properties"
      "zlim"    "axes_properties"
      "zoom"    "zoom_rect"        // + unzoom
    ]
    // Queries matching a scilab item but with a different meaning in an external 
    //  language. There are considered only if the priority target is external package
    if grep(options, "x")~=[] then
        e = [ e
      "contour"  "contour2d"
      "conv"    "convol"
      "dos"     "unig_g"
      "end"     "dollar"
      "home"    "tohome"
      "load"    "loadmatfile"
      "null"    "kernel"
      "pause"   "halt"
      "quiver"  "champ"         // quiver implemented in plotlib
      "system"   "unix_g"
      "type"    "unix_w"
      "ver"     "getversion"
      ]
    end
    ie = find(e(:,1)==convstr(item(1)))
    if ie~=[]
        rep = e(ie(1),2)
    else
        rep = item
    end

endfunction

// -------------------------------------------------------------------------

// Copyright (C) - 2015 - Samuel GOUGEON
// Utility function for uman()
function table = get_atoms_forge()
    table = [
    "accsum"	"accsum"	""
    "AdaptiveOptics"	""	"http://sciao.sourceforge.net/#authors"
    "ADS_CoLiSyS"	""	""
    "aerospace-toolbox"	"aerospace-toolbox"	""
    "aerospace_blockset"	"aerospace-blockset"	""
    "ampl_toolbox"	""	"http://code.google.com/p/scilab-mip"
    "Analog_and_DigitalCommunication"   ""  "http://www.geocities.ws/senthilirtt/"
    "Anfislab"  ""  "http://www.tritytech.com"
    "ANN_Toolbox"	"ann-toolbox"	""
    "apifun"	"apifun"	""
    "ardronemodule" ""  "https://code.google.com/p/ardronetoolbox/"
    "arduino"	"arduino"	"http://www.scilab-enterprises.com/"
    "arfit"	""	"http://www.gps.caltech.edu/~tapio/arfit/"
    "assert"	"assert"	""
    "BackDoor"	"BackDoor"	""
    "Cardiovascular"	""	"http://www.inria.fr/en/teams/sisyphe"
    "casci"	"casci"	"http://philippe.castagliola.free.fr"
    "CCA"	""	"http://people.ok.ubc.ca/ylucet/cca.html"
    "celestlab"	"celestlab"	""
    "cglab"	"cglab"	""
    "CLUSTER"	""	"http://www.xing.com/profile/Harald_Galda"
    "condnb"	"condnb"	""
    "Conint"	"conint"	""
    "conmin"	""	"http://code.google.com/p/scilab-mip"
    "convertlatin"	"convertlatin"	""
    "coselica"	"coselica"	""
    "CMA-ES"    ""  "http://www.cfuttrup.com"
    "CPGE"	"cpge"	""
    "csv_readwrite"	"csv-readwrite"	""
    "Curl" ""   ""
    "CUTEr"	"cuter"	""
    "dace_scilab"	"dace-scilab"	""
    "dbldbl"	"dbldbl"	""
    "dde_toolbox"	"dde-toolbox"	""
    "DD_QD"	""	"http://www.mi.kagu.tus.ac.jp/qupat.html"
    "DeCaToKi"	""	"http://sites.google.com/site/ronaystein/"
    "diffcode"	"diffcode"	""
    "digimetrie_Tools"	""	"http://www.digimetrie.com/"
    "dispmat"	"dispmat"	""
    "distfun"	"distfun"	""
    "Dynpeak"	""	""
    "edflib"	""	"http://www.teuniz.net/edflib/"
    "emd_toolbox"	""	"http://perso.ens-lyon.fr/patrick.flandrin/emd.html"
    "EMTTOOL"	""	""
    "FACT"  ""  ""
    "femtruss"	""	"http://code.google.com/p/scilab-mip/"
    "Financial"	"Financial"	""
    "fitters"	""	""
    "floatingpoint"	"floatingpoint"	""
    "fmincon"	"fmincont"	""
    "fsmtbx"	""	"http://fsmtbx.origo.ethz.ch/"
    "getDmatrix"    ""  "http://uolp.unifor.br/oul/pages/academico/posgraduacao/novoSite/professorPL.jsp?Professor=3820065&p_tipo_pagina=grad"
    "Global_Optim_toolbox"	""	"http://www.sc.iitb.ac.in/~bhagyesh/Scilab/GlobalOptimizationToolbox.html"
    "grocer"	""	"http://dubois.ensae.net/grocer.html"
    "guibuilder"	""	"http://www.tritytech.com/"
    "guimaker"	"guimaker"	""
    "helptbx"	"helptbx"	""
    "huffcomp"	""	""
    "hypt"  "hypt"  ""
    "hyb_auto_module"	"hyb-auto-module"	""
    "HYDROGRv50"	""	"http://lerat.julien.free.fr/CONT_FR/HYDROGR_FR.html"
    "ica_toolbox"	""	"http://research.ics.tkk.fi/ica/fastica/"
    "identification"	""	""
    "imsls"	"imsls"	""
    "indinf"	"indinf"	""
    "intprbs"	"intprbs"	""
    "iodelay"	""	""
    "IPD"	"IPD"	"http://www.xing.com/profile/Harald_Galda/"
    "ipoptlib"	"ipoptlib"	""
    "IPT2"	""	"http://cdac.in"
    "IPT3"  ""  ""
    "IWT"   ""  "http://www.cdac.in"
    "JIMS"	"JIMS"	""
    "json"	"json"	""
    "krisp"	""	""
    "lcc_windows"	"lcc-windows"	""
    "leftwavedenoise"   ""  "http://cas.ensmp.fr/~chaplais/fr/Ondelettes/Ondelettes/DebruitageOndelettes.html"
    "libsvm"	"libsvm"	""
    "linalg"	"linalg"	""
    "lolimot"	"lolimot"	"http://code.google.com/p/scilab-mip"
    "lowdisc"	"lowdisc"	""
    "lsitbx"	""	"http://lsitbx.origo.ethz.ch/"
    "makematrix"	"makematrix"	""
    "maple2scilab"	"maple2scilab"	""
    "Mascaret"  ""  ""
    "Mathieu"	"mathieu"	""
    "MatrixMarket"	"matmark"	""
    "Maxplus"	""	"(error)"
    "maxplus_petrinet"	""	""
    "Max_Plus_Algebra"	""	"http://www.its.ac.id/personal/index.php?id=Subiono"
    "MDPtoolbox"	""	"http://www.inra.fr/mia/T/MDPtoolbox"
    "MED"   "MED"   ""
    "metanet"	"metanet"	""
    "mi"	""	"http://penglab.janelia.org/proj/mRMR/"
    "microwave"	""	"http://www.reveyrand.fr/uW.html"
    "mingw"	"MinGW-toolbox"	""
    "minphase_toolbox"	""	""
    "mkltools"	"mkltools"	""
    "moc_toolbox"   "moc-toolbox"   ""
    "modbus"	"modbus"	""
    "module_lycee"	"module-lycee"	""
    "montesci"	""	"http://www.drivehq.com/web/ka_eins/montesci/"
    "MPScilab"	"mpscilab"	""
    "nan"	"nan-toolbox"	""
    "NARVAL"	""	"http://wwwfr.uni.lu/snt/"
    "NIDAQ"	"nidaq"	""
    "NISP"	"nisp"	""
    "nistdataset"	"nistdataset"	""
    "Nqueens"	""	""
    "NTG"	""	"http://wwwfr.uni.lu/interdisciplinary_centre_for_security_reliability_and_trust/people/foued_melakessou"
    "number"	"number"	""
    "OFIP"  ""  ""
    "opc_client"	"opc-client"	""
    "OpenRTDynamics"	""	""
    "opti"	""	""
    "optkelley"	"optkelley"	""
    "Ortpol"    "ortpol"  ""
    "Overload_Toolbox"	"overload"	""
    "PIMS"	"pims"	""
    "plotlib"	"plotlib"	""
    "portaudio"	"portaudio"	""
    "predictivemodule"  ""  ""
    "ProActiveConnector"	""	"http://proactive.activeeon.com"
    "PSO"	"pso-toolbox"	""
    "PVM"	"pvm"	""
    "quapro"	"quapro"	""
    "rdataset"	"rdataset"	""
    "regtools"	"regtools"	""
    "rest_client"	""	""
    "RLDesign"	""	""
    "rltool"	""	"http://wwwa.mpi-magdeburg.mpg.de/people/ishan/scilab/rltool.htm"
    "Robotics_Toolbox"  ""  ""
    "scetoexe"	"scetoexe"	""
    "Scholar_Lycee_in_English"	"module-lycee"	""
    "scibench"	"scibench"	""
    "scicheckupdate"	"scicheckupdate"	""
    "scicobyla"	"scicobyla"	""
    "SciCrypt"	"scicrypt"	""
    "SciCSDP"	""	"http://code.google.com/p/scilab-mip/"
    "scidb"	"scidb"	""
    "scidemo"	"scidemo"	""
    "scidoe"	"scidoe"	""
    "sciFLT"	""	"http://www.geocities.ws/jaime_urzua/sciFLT/sciflt.html"
    "SciFreeFEM"	""	"http://code.google.com/p/scilab-mip/"
    "SciGLPK"	""	"http://code.google.com/p/scilab-mip/"
    "sciGPGPU"	""	""
    "scilab2c"	"scilab2c"	""
    "Scilab_SSE"	""	"http://www.scilab.org"
    "Scilab_XLL"	"scilab-xll"	""
    "scilib_geo"	""	"http://scilib-geo.area7.de/"
    "SciLPSolve"	""	"http://code.google.com/p/scilab-mip/"
    "scinlopt"	""	"http://code.google.com/p/scilab-mip"
    "sciopustc"	""	"http://opus-project.fr/"
    "scipad"	""	"http://sourceforge.net/projects/scipad/"
    "sci_ipopt"	"sci-ipopt"	""
    "Scholar_Lycee_in_English"  ""  "http://scilab-scholar.blogspot.com"
    "serial"	""	"github.com/sengupta/Scilab-Serial"
    "serialport"	""	""
    "SIMM"	"SIMM"	""
    "simplex"	""	"http://code.google.com/p/scilab-mip"
    "SIP"	"SIP"	"http://siptoolbox.sourceforge.net"
    "siseli"	""	"http://www.askrprojects.net/software/siseli/index.html"
    "SIVP"	""	"http://sivp.sourceforge.net/"
    "SMP"	"smp"	""
    "sndfile_toolbox"	"sndfile-toolbox"	""
    "socket-server" ""  "https://github.com/ishit/scilab-network"
    "socket_toolbox"	""	"http://www.reveyrand.fr/SOCKET2.html"
    "specfun"	"specfun"	""
    "toolbox_spgl1" "toolbox-spgl1"  ""
    "spilu"	"spilu"	""
    "splspc"	"splspc"	""
    "SSP_Toolbox"	""	"www.cdackolkata.in"
    "Statpack"	"Statpack"	""
    "stftb"	"stftb"	""
    "stixbox"	"stixbox"	""
    "sudoku"	"sudoku"	""
    "supportvectormachine"  ""  ""
    "swt"	"swt"	""
    "texmacs"	"texmacs"	""
    "Two-Dimensional_Transformations"	""	"www.cdackolkata.in"
    "uman"	""	""
    "uncprb"	"uncprb"	""
    "usb_toolbox"	"usb-toolbox"	""
    "Wavelab"	""	"http://www-stat.stanford.edu/~wavelab/Wavelab_850/index_wavelab850.html"
    "WriteXmlExcel"	"writexmlexcel"	""
    "XCPL"	""	""
    "xls_link"	"xls-link"	""
    "xmllab"	"xmllab"	"http://xmllab.org"
    "xmltodocbook"	"xmltodocbook"	""
    ]
endfunction
// ===========================================================================

// Copyright (C) - 2015 - Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// Its terms are available at
// http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
// 
// Utility function for uman()
function resu = uman_strsubst(text, pattern, replace, option)
    // option: "r" if the pattern is a regular expression (with delimiters and options)
    // replace: Group id: with TCL version = "\#"    with Java = "$#"
    Sclass = "java.lang.String"

    [lhs, rhs] = argn()

    if rhs>2 then
        if text==[] then
            resu = []
            return
        end
        
        sizeText = size(text)

        // Chosing the Java method: litteral, regexp, regexp once
        // ------------------------------------------------------
        if rhs>3 then
            if option=="r" then
                jmeth = 1  // "replaceAll"
            elseif or(option==["ro" "or"])
                jmeth = 3  // "replaceFirst"
            else
                msg = _("%s: Wrong value for input argument #%d: Must be in the set {%s}.\n")
                error(msprintf(msg, "strsubst", 4, "''r'',''ro''"))
            end
        else
            jmeth = 2  // "replace"
            option = ""
        end
        
        // Compiling the meta-method
        // -------------------------
        try
            arrayRep = jnewInstance("arrayRep")
        catch
            jimport(Sclass)
            arrayRep = jcompile("arrayRep", ..
            ["public class arrayRep {"
             "public static String[] replaceInArray(int meth, String arr[], String[] pat, String[] rep) {"
             "    int nbelem = arr.length; "
             "    String tmp;"
             "    String[] out = new String[nbelem];"
             "    if (meth==1) { "
             "    for (int i = 0; i < nbelem; i++) { "
             "        tmp = arr[i]; "
             "        for (int p = 0; p < pat.length; p++) { "
             "            tmp = tmp.replaceAll(pat[p], rep[p]); "
             "            } "
             "        out[i] = tmp; "
             "        } "
             "    } "
             "    else if (meth==2) {"
             "    for (int i = 0; i < nbelem; i++) { "
             "        tmp = arr[i]; "
             "        for (int p = 0; p < pat.length; p++) { "
             "            tmp = tmp.replace(pat[p], rep[p]); "
             "            } "
             "        out[i] = tmp; "
             "        } "
             "    } "
             "    else {"
             "    for (int i = 0; i < nbelem; i++) { "
             "        tmp = arr[i]; "
             "        for (int p = 0; p < pat.length; p++) { "
             "            tmp = tmp.replaceFirst(pat[p], rep[p]); "
             "            } "
             "        out[i] = tmp; "
             "        } "
             "    } "
             "    return out; "
             "    }"
             "}"]);
         end
        //pattern = stripblanks(pattern)

        // Pre-processing the patterns: Java does not accept "/../modifiers" syntax.
        //  - modifiers must be inserted as (?#) switches in head of regexp
        //  - delimiters are not expected
        if option~="" then
            // http://docs.oracle.com/javase/tutorial/essential/regex/pattern.html
            // Extracting and checking delimiters:
            delim = part(pattern, 1)
            //tmp = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
            //forbid = strsplit(tmp)'
            forbid = ["a" "b" "c" "d" "e" "f" "g" "h" "i" "j" "k" "l" "m" "n" ..
                      "o" "p" "q" "r" "s" "t" "u" "v" "w" "x" "y" "z" ..
                      "A" "B" "C" "D" "E" "F" "G" "H" "I" "J" "K" "L" "M" "N" ..
                      "O" "P" "Q" "R" "S" "T" "U" "V" "W" "X" "Y" "Z" ..
                      "0" "1" "2" "3" "4" "5" "6" "7" "8" "9"]
            nforbid = sum(grep(delim, forbid))
            if nforbid>0 then
                msg = _("%s: Delimiters must be non alphanumeric (%d pattern(s) concerned)")
                error(msprintf(msg, "strsubst", nforbid))
            end
            // Checking the closing delimiter and extracting modifiers:
            modifiers = strrchr(pattern, delim) // should include the closing delimiter
            Nunclosed = length(find((modifiers==pattern)))
            if Nunclosed>0
                msg = _("%s: the delimiter in the pattern is sometimes unclosed (%d pattern(s) concerned)")
                error(msprintf(msg, "strsubst", Nunclosed))
            end
            // Supported modifiers:
            // u => (?u) : unicode case: set by default
            // i => (?i) : case insensitive
            // m => (?m) : multiline
            // x => (?x) : ignores comments starting with # until the end of the line
            // s => (?s) : "." matches also EOL
            Modifiers = emptystr(modifiers)+"(?u"
            for m = ["i" "m" "x" "s"]
                im = grep(modifiers,m)
                Modifiers(im) = Modifiers(im)+m
            end
            Modifiers = Modifiers + ")"
            pattern = strncpy(pattern, length(pattern)-strcspn(strrev(pattern), delim)-1)
            L = length(pattern)
            for i = 1:length(L)
                pattern(i) = Modifiers(i) + part(pattern(i), 2:L(i))
            end
        end

         // Formating argins
         // ----------------
        text    = text(:)'
        pattern = pattern(:)'
        replace = replace(:)'
        if  size(pattern,"*")==1 then
            replace = replace(1)
        elseif size(replace,"*")==1
            replace = emptystr(pattern) + replace
        elseif size(replace,"*")~=size(pattern,"*")
            msg = _("%s: Incompatible input arguments #%d and #%d: Same sizes expected.\n")
            error(msprintf(msg, "strsubst", 2, 3))
        end
        
        // Calling the routine
        // -------------------
        resu = arrayRep.replaceInArray(jmeth, text, pattern, replace)
        
        // Post-formatting the result
        resu = matrix(resu, sizeText)

    else  // Inline examples with strsubst()
          // -------------------------------
        todo = [_("// Literal substitution. Unicode chars supported:")
                "strsubst(sc, ""l"", ""L"")"
                _("// with matrices of Patterns and respective Replacements:")
                "strsubst(sc, [""l"" ""α""], [""L"" ""A""]) "
                _("// Regular expression (regexp), ""ro""= Replace Once: only the first match is replaced:")
                "strsubst(""abcABCabc"", ""/[^^]a/i"", ""*"", ""ro"")"
                _("// Regexp, all occurrences replaced. /i = case insensitive. /imxs supported: ")
                "strsubst(sc, ""/(l+)/i"", ""é"", ""r"")"
                _("// Regexp, with groups replacements:")
                "strsubst(sc, ""/(.?)(l+)/i"", ""$2$1"", ""r"")"
                ]
        T = "sc = [""Hello"" ""Hólà"" ""ALLO"" ""Γεια σας""];"
        write(%io(2),">> "+T)
        execstr(T)
        for ex = todo'
            if part(ex,1:2)~="//" then
                write(%io(2), ">> "+ex)
                execstr("r="+ex)
                disp(r)
            else
                write(%io(2), ascii(10)+ex)
            end
        end
        resu = ""
    end
endfunction
// --------------------------------------------------------------------------

// Copyright (C) - 2013 - Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// Its terms are available at
// http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// TCL implementation: supports Scilab < 5.5. Mac OS not supported

//function out = uman_strsubst(text, pattern, replace, option)
//    // option : unused here (for compatibility)
//	// http://bugzilla.scilab.org/12808
//    pattern0 = pattern
//    replace0 = replace
//    if size(pattern,"*")==1 then
//        pattern = "set RE {" + pattern + "}"
//        replace = "set RP {" + replace(1) + "}"
//        TCL_EvalStr(pattern)
//        TCL_EvalStr(replace)
//        out = text
//        for i = 1:size(text,"*")
//            texte = "set TXT {" + text(i) + "}"
//            try
//                TCL_EvalStr(texte)
//            catch
//                if text(i)=="\"
//                    TCL_EvalStr("set TXT {\\}")
//                else
//                    write(%io(2), lasterror())
//                    whereami()
//                    return
//                end
//            end
//            out(i) = TCL_EvalStr("regsub -all $RE $TXT $RP")
//            // http://www.tcl.tk/man/tcl8.6/TclCmd/regsub.htm
//        end
//    else
//        for i = 1:min([size(pattern,"*") size(replace,"*")])
//            text = uman_strsubst(text, pattern(i), replace(i), "r")
//        end
//        out = text
//    end
//    //t = emptystr(100,100) + html;
//    //tic(),uman_strsubst(t,"(<.*?>)","");toc() 
//    //// vectorized with strcat() + processing + strsplit()   = 153 s
//    //// vectorized with a for loop                           = 38.8s
//endfunction
// =========================================================================

//
// FIXED & DONE:
//
//  1.01 (2015-03-27)
//  ----
//  * Help pages updated
//  * ja_JP: Ends of sentences were not wrappable points
//  * summaries : wrapping was only partial. left justification of the unique
//     wrapped line could be wrong.
//  * "uman openserial" displayed an error
//  * "uman type" returned an error (after the implementation of the 
//     case-sensitivity management)
//  * Some error messages returned by the OS could be displayed
//  * Automatical refreshing / reloading was done once a day. Now once a week.
//  * The pattern was always case-insensitive. Now only when all in lowercase.
//  * when the pattern was finally head_commented, head_comment was displayed twice
//  * under Linux, jar files were not always properly detected & listed 
//
//  1.0
//  ---
//  * The toolbox was not completly set
//  * There were no unitary tests (.dia.ref still to be writen)
//  * help in head_comments was not up-to-date
//  * searching the pattern was case-sensitive
//  * "L.." option was possible only as last option
//  * Itemized lists: wrapped lines were not indented
//  * Itemized lists: some lines were wrongly wrapped
//  * Examples with strings including some tages were not displayed
//  * in some pages, some %26#8212 raw entities appeared.
//  * The list of ATOMS modules (and related forge or website) was not up-to-date
//    (2015-03-20. Last = openCV Function implementation...)
//  * Summary: the path within the help was not displayed
//  * Option "a" added : All sections displayed (but bugs URLs)
//  * The title of refsections could have extra spaces.
//  * There was no help page in french
//  * "x" mode: non existing pattern did not display the 
//     "uman: "<pattern>" is not documented" message
//  * ~"wb" mode: braces "{" and "}" were not converted into their equivalent "[" "]" 
//  * "wb": symbols were not regexp-escaped before calling bugzilla search in 
//      regexp mode.
//  * "x" : priority to external patterns was dropped, since redirections are
//     called only in second. 
//  * "wb": any %# pattern prefixed with % was wrongly translated into percent#
//     before calling the online bug page 
//  * redirection: some online help or bug pages opened targetting the 
//     original pattern instead of the redirected one.
//  * "swb" opened online help and bug pages. Now "s" kills "w" and "b"
//  * "wb": when the pattern was not found, Scilab help page AND bugzilla page
//     were addressed together, instead of only Bugzilla
//  * "wb": when the pattern was not found, Scilab help page was addressed 
//     instead of the bugzilla one.
//  * redirections were called a priori, instead of only when the initial 
//    pattern was not found
//  * ATOMS modules: Many forge references were not correct. ALl checked.
//  * Tip and Caution tags were not displayed (only Note and Warning)
//  * "h" option introduced for history info
//  * "h" option (help browser) renamed into "g" (GUI). This history also updated
//  * <code> style was not reviewed and improved. 
//  * sections identification and display refactorized. For multiple sections 
//    of the same type, only the first occurence was displayed
//  * Some WARNING and NOTE were not detected
//  * In pieces of code, "#;#" style was allowed, instead of "#; #"
//  * There was a memory leak due to undeleted xml documents
//  * <code>..</code> sections could be wrapped
//  * options: when flags were used before and after the item and "L" was used,
//     most of options were ignored.
//  * Tables: missing cells yielding too short lines yielded an error.
//  * Tables: rows with only one cell were roughly processed. Now, their complex
//     content is processed as for other sectionning tags.
//  * Some <pre> sequences of programlisting were not processed correctly: their
//     id was displayed instead of the code. Example: uman fft cdLfr  in the
//     "Syntaxe longue pour la FFT mutidimensionnelle" section
//  * Mac OS: file finding implemented. Mac OS should now be supported (to be tested)
//  * German added for sections detection
//  * last implementations were no longer working under Linux. Repared
//  * xml page updated
//  * option "p" (package) renamed "x" (eXternal) 
//  * strsubst2() implemented through TCL did not support Mac OS
//    => replaced with the Java version
//  * patterns including spaces did not convert then into "_" (as in many
//     "properties" entries.
//  * Redirections : input elements (left column) were not considered whether
//     they had uppercase characters.
//  * External faux amis were not reachable. For instance, the equivalence for
//    Octave "end" was not accessible (since "end" exists in SCilab). They are
//    now reachable using the same "x" option. 
//  * "wb" on scilab functions: plot2d or plot3d were in the list for "plot"
//  * For pages presenting several functions or items, only explicitly
//     declared ones were supported with some explicit redirections.
//    They are now all supported without explicit declared redirections.
//  * The list of JAR was set each time uman was called. It is now saved, 
//     and "r" refreshes it.
//  * When redirecting, it was not possible to force some options othen than "s"
//  * external equivalences were scanned only when the primary query did not 
//     match anything. They are now scanned a priori, in order to enable
//     special processings as forcing the "s" flag for some actual queries.
//  * when the "property" or "properties" string was in the query, the display
//    of the "description" part was not forced, contrarily to what the help said.
//  * Some main categories were not easyly accessible: variables, hfd5, signal,
//     metanet, statistics, windows, xml, sparses, shortcuts, graphics, files, 
//     atoms
//  * opening default web browser under Linux was not supported
//  * "b" : if ATOMS without forge, the comments section of ATOMS page was not
//     targeted
//  * "man" renamed "uman"
//  * "x" excluded scanning Scilab pages => role modified
//  * <tbody>.. was not supported
//  * Summaries : alignements of "—" implemented
//  * Lines wrapping sometimes overflown for imbricated description lists: fixed!
//  * support to <note> and <warning> added
//  * atoms/forge data were in an external file. Now in a function
//  * <em>..</em> appeared as is in the text
//  * option "x" (eXternal packages first) implemented
//  * option "f" (force) renamed "r" (reload)
//  * help page in en_US: written and compiled
//  * language id in options could be considered as common options.
//  * "g" management improved: 
//      - loading status of contrib modules was not defined
//      - unloaded ATOMS modules were temptatively called to helpbrowser
//  * option "r" was not proposed
//  * not ATOMS modules in SCI\contrib and other external modules in 
//    SCIHOME\contrib were not detected 
//  * ATOMS modules not set for ALL USERS were not correctly processed
//  * the "s" option did not work for ATOMS modules
//  * extraction zip sous Linux
//  * toolbox : setup (to be finished)
//  * "s" was not available => added
//  * uman patconvert  yielded an error (due to strsubst2: set TXT {\} )
//  * for ATOMS modules, the web target designed for "wb" options was not correct. 
//  * "w" option used for a function of an external module was unsuccessful: 
//     there is no online help for these modules => replaced with console display
//  * Only the first GET's separators "&" in URLs  was replaced. Following ones
//     were still blocking XML parsing. Example: uman conjgrad c
//  * Description lists: indentation depth was too large. It made line-wrapping
//    mis-working. Examples:
//      uman schur cd
//      uman fft cdlfr   // largeur description incorrecte
//  * when a page describes several functions and the short description of one
//     of them was missing, an error occurred. 
//     Example: uman mprintf cd    See http://bugzilla.scilab.org/13748
//  * The diary was stopped with the option "j" even when the web was targetted
//  * "w": when the item was not found in Scilab, head_comments was called instead
//    of calling the online search page.
//  * "wb": when the item was not found in Scilab, the item was not passed to
//      the Bugzilla search engine.
